/**
 *  @file    middle-code.h
 *  @author  Guillaume Bour. 2002
 
 *  @version 0.1
 *    @date  01/02/2002

 *  intermediate code management (indirects triplets)
 */
#ifndef _MIDDLE_CODE_H_
#define _MIDDLE_CODE_H_

#include <gtk/gtk.h>
#include "op.h"
#include "instr.h"
#include "type.h"

/*>>> arguments type                                                         */
typedef enum {
  MC_NONE,           /*>>> no argument                                       */
  MC_VAR,            /*>>> variable (of type struct s_variable *)            */
  MC_LIT,            /*>>> literal  (of type struct s_literal  *)            */
  MC_REF,            /*>>> other instruction (of type struct s_instr *)      */
  MC_LIST            /*>>> list of mc's      (of type struct s_mc *)         */
} mc_t;

/*>>> middle-code element structure                                          */
struct s_mc {
  mc_t t_arg;     /*>>> argument type: instruction, variable or literal      */
  gpointer arg;   /*>>> argument value                                       */
};


/*>>> functions prototypes                                                  */
struct s_mc *mc_alloc(mc_t, gpointer);
void mc_free(struct s_mc *);
inline type_t MC_GET_ARG_TYPE(struct s_mc *);
void mc_init(void);
gboolean mc_beginning_procedure(gchar *);
void mc_ending_procedure(void);
gboolean mc_add_proc_parameter(gchar *);
struct s_mc *mc_add_literal(type_t, ...);
struct s_mc *mc_add_variable(gchar *);
struct s_mc *mc_add_noarg_op(gboolean, op_t);
struct s_mc *mc_add_unary_op(gboolean, op_t, struct s_mc *);
struct s_mc *mc_add_binary_op(gboolean, op_t, struct s_mc *, struct s_mc *);
void mc_update_branch();
gpointer mc_find_procedure(gchar *);

#endif /* _MIDDLE_CODE_H_ */
