
#include <math.h>
#include <time.h>
#include <gtk/gtk.h>
#include "turtle.h"
#include "middle-code.h"
#include "instr.h"
#include "execution.h"
#include "proc.h"
#include "literal.h"
#include "debug.h"

#ifndef G_PI
#  define G_PI   M_PI
#endif

extern struct s_turtle my_turtle;

/*>>> operations callbacks         */
typedef guint(*op_clb_t)(struct s_instr *);
op_clb_t op_clb[44] = {
  clb_noop,
  clb_proccal,
  clb_return,
  clb_moveforward,
  clb_moveback,
  clb_turnright,
  clb_turnleft,
  clb_setpos,
  clb_setheading,
  clb_clearscreen,
  clb_hideturtle,
  clb_show_turtle,
  clb_hidepen,
  clb_showpen,
  clb_setpredefcolor,
  clb_setuserdefcolor,
  clb_notrace,
  clb_trace,
  clb_write,
  clb_stop,
  clb_affect,
  clb_locaffect,
  clb_dance,
  clb_sleep,
  clb_getheading,
  clb_hazard,
  clb_repeat,
  clb_loopcnt,
  clb_if,
  clb_else,
  clb_while,
  clb_jump,
  clb_and,
  clb_or,
  clb_eq,
  clb_lt,
  clb_gt,
  clb_leq,
  clb_geq,
  clb_add,
  clb_sub,
  clb_mul,
  clb_div,
  clb_uneg
};

/*>> predefined colors table  */
gushort predefined_colors[16][3] = {
  {     0,     0,     0}, // 0 black
  {     0,     0, 65535}, // 1 blue
  {     0, 65535,     0}, // 2 green
  {     0, 65535, 65535}, // 3 cyan
  { 65535,     0,     0}, // 4 red
  { 65535,     0, 65535}, // 5 magenta
  { 65535, 65535,     0}, // 6 yellow
  { 65535, 65535, 65535}, // 7 white
  { 39835, 24672, 15163}, // 8 brown
  { 50629, 34952,  4626}, // 9 light-brown
  { 25700, 41634, 16448}, //10 mid-green
  { 30840, 48059, 48059}, //11 blue-green
  { 65535, 38293, 30583}, //12 salmon
  { 37008, 29041, 53456}, //13 blue-ish
  { 65535, 41891,     0}, //14 orange
  { 47031, 47031, 47031}  //15 or
};


void exec_all()
{
  guint pos;

  //#ifndef DEBUG2
  gtk_notebook_set_page(GTK_NOTEBOOK(my_turtle.notebook), 0);
  //#endif

  debug_append("*** Starting execution\n");
  exec_execute_proc(my_turtle.main_proc);
  debug_append("*** Ending execution\n");

}

void exec_execute_proc(struct s_proc *proc)
{
  register guint cnt, newcnt;
  GSList *curelem = proc->instructions;
  register struct s_instr *instr;

  for(cnt = 0; curelem != NULL;)
    {
      instr = (struct s_instr *) curelem->data;

      printf("instr: %d\n", instr->op);
      newcnt = (* op_clb[instr->op])(instr);

      if(newcnt != -1)
	{
	  cnt = newcnt;
	  curelem = g_slist_nth(proc->instructions, cnt);
	} else {
	  cnt++;
	  curelem = g_slist_next(curelem);
	}
    }
}




/**
   Drawing a simply dot.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. a dot is a box of 3 pixel width and height.
	   
   @return  <none>
 */
inline void draw_turtle_dot()
{
  GdkRectangle update_rect;
  GdkGC *gc;
  GdkColormap *colormap;

  update_rect.x = (int) (my_turtle.x - 1);
  update_rect.y = (int) (my_turtle.y - 1);
  update_rect.width = 3;
  update_rect.height = 3;

  gc = gdk_gc_new(my_turtle.pixmap);
  colormap = gtk_widget_get_colormap(my_turtle.drawing_field);
  gdk_colormap_alloc_color(colormap, &(my_turtle.pencolor), TRUE, TRUE);

  //gc = gdk_gc_new(my_turtle.drawing_field);
  gdk_gc_set_foreground(gc, &(my_turtle.pencolor));

  gdk_draw_rectangle(my_turtle.pixmap,
		     //my_turtle.drawing_field,
		     //my_turtle.gc,
		     gc,
		     TRUE,
		     update_rect.x, update_rect.y,
		     update_rect.width, update_rect.height);

  gtk_widget_draw(GTK_WIDGET(my_turtle.drawing_field), &update_rect);
  gdk_gc_destroy(gc);
}









/**
   Instruction 'CLEAR SCREEN': clearing screen.<br>

   <b>visibility :: <i>public</i></b>
   
   A COMPLETER
   EST-CE QU'ON REINITIALISE LA TORTUE??

   @param   instr                       the instruction     
   @return  -1
 */
guint exec_clear_screen(struct s_instr *instr)
{
  return(-1);
}


/**
   Instruction 'HIDE TURTLE': hiding turtle.<br>

   <b>visibility :: <i>public</i></b>
   
   A COMPLETER

   @param   instr                       the instruction     
   @return  -1
 */
guint exec_hide_turtle(struct s_instr *instr)
{
  return(-1);
}


/**
   Instruction 'SHOW TURTLE': showing turtle.<br>

   <b>visibility :: <i>public</i></b>
   
   A COMPLETER

   @param   instr                       the instruction     
   @return  -1
 */
guint exec_show_turtle(struct s_instr *instr)
{
  return(-1);
}



/**
   Instruction 'SET USERDEF COLOR': changing pen color for user defined color.
   <br>

   <b>visibility :: <i>public</i></b>
   
   @param   instr                       the instruction     
   @return  -1
 */
guint exec_set_userdefcolor(struct s_instr *instr)
{
  /*  my_turtle.pencolor.red   = (int) GPOINTER_TO_UINT(instr->arg1) * (63536/256);
  my_turtle.pencolor.green = (int) GPOINTER_TO_UINT(instr->arg2) * (63536/256);
  my_turtle.pencolor.blue  = (int) GPOINTER_TO_UINT(instr->arg3) * (63536/256);
  */
  return(-1);
}






/**
   Instruction 'NOOP': do nothing.<br>

   <b>visibility :: <i>public</i></b>

   @param   instr                       the instruction     
   @return  -1
 */
guint clb_noop(struct s_instr *instr)
{
#ifdef DEBUG2
  //!pas bon
  guint pos = gtk_text_get_length(GTK_TEXT(my_turtle.debug_txt)); 

  gtk_editable_insert_text(GTK_EDITABLE(my_turtle.debug_txt), 
			   "  # executing clb_noop()\n", 25, &pos);
#endif

  return(-1);
}


/**
   Instruction 'ENTER PROC': entering procedure.<br>

   <b>visibility :: <i>public</i></b>
   
   THE SECOND ARG WILL BE A LIST OF VALUES

   @param   instr                       the instruction     
   @return  -1
 */
guint clb_proccal(struct s_instr *instr)
{
  struct s_proc *oldproc,
    *procptr =(struct s_proc *)instr->arg1->arg;

  oldproc = my_turtle.cur_proc;
  my_turtle.cur_proc = procptr; 
  exec_execute_proc(procptr);

  my_turtle.cur_proc = oldproc;

  return(-1);
}

guint clb_return(struct s_instr *instr){}

/**
   Instruction 'MOVE FORWARD': drawing a line.<br>

   <b>visibility :: <i>public</i></b>

   @param   instr                       the instruction     
   @return  <none>
 */
guint clb_moveforward(struct s_instr *instr)
{
  type_t type;
  guint i, dist;
  struct timespec tm;
#ifdef DEBUG2
  debug_append("  # executing clb_moveforward()\n");
#endif
  
  tm.tv_sec = 0;
  tm.tv_nsec = my_turtle.delay;

  type = MC_GET_ARG_TYPE(instr->arg1);
  if(type == T_INT)
    { dist = *(guint *)MC_GET_ARG_VALUE(instr->arg1); }
  else if(type == T_FLOAT)
    { dist = (guint) *(gdouble *)MC_GET_ARG_VALUE(instr->arg1); }
  
  printf("moveforward distance value: %d\n", dist);

  for(i = 0; i < dist; i++)
    {
      my_turtle.x += 2.0 * cos(my_turtle.angle);
      my_turtle.y += 2.0 * sin(my_turtle.angle);
      
      if(my_turtle.trace) 
	{ draw_turtle_dot(); }
      
      nanosleep(&tm, NULL);
    }

  return(-1);
}

/**
   Instruction 'MOVE BACKWARD': drawing a line (back stepping).<br>

   <b>visibility :: <i>public</i></b>

   @param   instr                       the instruction     
   @return  <none>
 */
guint clb_moveback(struct s_instr *instr)
{
  my_turtle.angle += 180.0;
  clb_moveforward(instr);
  my_turtle.angle += 180.0;  

  return(-1);
}


/**
   Instruction 'TURN RIGHT': turning right.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. angle value is stored in radians
   @param   instr                       the instruction     
   @return  <none>
 */
guint clb_turnright(struct s_instr *instr)
{
  register type_t type;
  register double angle;
#ifdef DEBUG2
  debug_append("  # executing clb_turnright()\n");
#endif

  type = MC_GET_ARG_TYPE(instr->arg1);
  if(type == T_INT)
    { angle = (gdouble) *(guint *)MC_GET_ARG_VALUE(instr->arg1); }
  else if(type == T_FLOAT)
    { angle = *(gdouble *)MC_GET_ARG_VALUE(instr->arg1); }

  my_turtle.angle = fmod((my_turtle.angle + angle * (G_PI/180)), (2 * G_PI));
  return(-1);
}

/**
   Instruction 'TURN LEFT': turning left.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. angle value is stored in radians
   @param   instr                       the instruction     
   @return  -1
 */
guint clb_turnleft(struct s_instr *instr)
{
  register type_t type;
  register double angle;
#ifdef DEBUG2
  debug_append("  # executing clb_turnleft()\n");
#endif

  type = MC_GET_ARG_TYPE(instr->arg1);
  if(type == T_INT)
    { angle = (gdouble) *(guint *)MC_GET_ARG_VALUE(instr->arg1); }
  else if(type == T_FLOAT)
    { angle = *(gdouble *)MC_GET_ARG_VALUE(instr->arg1); }

  my_turtle.angle = fmod((my_turtle.angle + 2 * G_PI - angle * (G_PI/180)), 
			 (2 * G_PI));

  return(-1);
}

guint clb_setpos(struct s_instr *instr){}
guint clb_setheading(struct s_instr *instr){}
guint clb_clearscreen(struct s_instr *instr)
{
  GdkGC *gc;
  GdkColormap *colormap;
  GdkRectangle update_rect;
  //GdkColor whitec;

#ifdef DEBUG2
  debug_append("  # executing clb_clearscreen()\n");
#endif

  update_rect.x = 0;
  update_rect.y = 0;
  update_rect.width = ((my_turtle.drawing_field)->allocation).width;
  update_rect.height = ((my_turtle.drawing_field)->allocation).height;

  //gc = gdk_gc_new(my_turtle.pixmap);
  //colormap = gtk_widget_get_colormap(my_turtle.drawing_field);
  //gdk_colormap_alloc_color(colormap, &(my_turtle.pencolor), TRUE, TRUE);

  //gc = gdk_gc_new(my_turtle.drawing_field);
  //whitec.red   = 65535; //predefined_colors[7][0];
  //whitec.green = 65535; //predefined_colors[7][1];
  //whitec.blue  = 65535; //predefined_colors[7][2];
  //gdk_gc_set_foreground(gc, &whitec);

  gdk_draw_rectangle(my_turtle.pixmap,
		     (my_turtle.drawing_field)->style->white_gc,
		     //gc,
		     TRUE,
		     update_rect.x, update_rect.y,
		     update_rect.width, update_rect.height);


  gtk_widget_draw(GTK_WIDGET(my_turtle.drawing_field), &update_rect);
  gdk_gc_destroy(gc);

  my_turtle.x = 0;
  my_turtle.y = 0;
  my_turtle.angle = 0;

  my_turtle.delay = 0; //50000000
  my_turtle.trace = TRUE;

  //default pen color: black
  my_turtle.pencolor.red = 0;
  my_turtle.pencolor.green = 0;
  my_turtle.pencolor.blue = 0;

  return(-1);
}

guint clb_hideturtle(struct s_instr *instr){}
guint clb_show_turtle(struct s_instr *instr){}
guint clb_hidepen(struct s_instr *instr){}
guint clb_showpen(struct s_instr *instr){}

/**
   Instruction 'SET PREDEF COLOR': changing pen color for predefined color.<br>

   <b>visibility :: <i>public</i></b>
   
   @param   instr                       the instruction     
   @return  -1
 */
guint clb_setpredefcolor(struct s_instr *instr)
{
  register guint index = *(guint *)MC_GET_ARG_VALUE(instr->arg1);

#ifdef DEBUG2
  debug_append("  # executing clb_setpredefcolor()\n");
#endif

  printf("setting predefined color: %d\n", index);
  my_turtle.pencolor.red   = predefined_colors[index][0];
  my_turtle.pencolor.green = predefined_colors[index][1];
  my_turtle.pencolor.blue  = predefined_colors[index][2];
  return(-1);
}

guint clb_setuserdefcolor(struct s_instr *instr){}

/**
   Instruction 'NO TRACE': hide pen trace during turtle moves.<br>

   <b>visibility :: <i>public</i></b>
   
   @param   instr                       the instruction     
   @return  -1
 */
guint clb_notrace(struct s_instr *instr)
{
#ifdef DEBUG2
  debug_append("  # executing clb_notrace()\n");
#endif

  my_turtle.trace = FALSE;
  return(-1);
}


/**
   Instruction 'TRACE': show pen trace during turtle moves.<br>

   <b>visibility :: <i>public</i></b>
   
   @param   instr                       the instruction     
   @return  -1
 */
guint clb_trace(struct s_instr *instr)
{
#ifdef DEBUG2
  debug_append("  # executing clb_trace()\n");
#endif

  my_turtle.trace = TRUE;
  return(-1);
}

guint clb_write(struct s_instr *instr){}
guint clb_stop(struct s_instr *instr){}
guint clb_affect(struct s_instr *instr){}
guint clb_locaffect(struct s_instr *instr){}
guint clb_dance(struct s_instr *instr){}
guint clb_sleep(struct s_instr *instr){}
guint clb_getheading(struct s_instr *instr){}
guint clb_hazard(struct s_instr *instr){}

/**
   Instruction 'LOOP': executing a loop.<br>

   <b>visibility :: <i>public</i></b>
   
   @remarks
            1. instr->curval containt the number of loops to do again + 1

   @param   instr                       the instruction     
   @return  -1
 */
guint clb_repeat(struct s_instr *instr)
{
  register guint loopcnt;

#ifdef DEBUG2
  debug_append("  # executing clb_repeat()\n");
#endif

  /* we can't have a loopcount equal 0(zero) because the value NULL is also 0.
     so we give a loopcount+1 (always > 0)
  */
  if(instr->value == NULL)
    { 
      register type_t type;
      register guint value;

      type = MC_GET_ARG_TYPE(instr->arg2);
      if(type == T_INT)
	{ value = *(guint *)MC_GET_ARG_VALUE(instr->arg2); }
      else if(type == T_FLOAT)
	{ value = (guint) *(gdouble *)MC_GET_ARG_VALUE(instr->arg2); }

      instr->value = GUINT_TO_POINTER(value + 1); 
    }

  loopcnt = GPOINTER_TO_UINT(instr->value);
  printf("loopcnt: %d\n", loopcnt);
  
  if(loopcnt == 1)
    { 
      guint index;

      index = g_slist_index((my_turtle.cur_proc)->instructions, 
			    instr->arg1->arg);
      printf("index: %d - %d\n", instr->arg1->arg, index);
      instr->value = NULL;
      
      return(index); 
    }
  
  instr->value = GUINT_TO_POINTER(--loopcnt);
  return(-1);
}



guint clb_loopcnt(struct s_instr *instr){}
guint clb_if(struct s_instr *instr){}
guint clb_else(struct s_instr *instr){}
guint clb_while(struct s_instr *instr){}

/**
   Instruction 'JUMP': executing an unconditionnal jump.<br>

   <b>visibility :: <i>public</i></b>
   
   @param   instr                       the instruction     
   @return  -1
 */
guint clb_jump(struct s_instr *instr)
{
  guint index;

#ifdef DEBUG2
  debug_append("  # executing clb_jump()\n");
#endif

  index = g_slist_index((my_turtle.cur_proc)->instructions, instr->arg1->arg);

  printf("jump to: %d - %d\n", instr->arg1->arg, index);

  return(index);
}

guint clb_and(struct s_instr *instr){}
guint clb_or(struct s_instr *instr){}
guint clb_eq(struct s_instr *instr){}
guint clb_lt(struct s_instr *instr){}
guint clb_gt(struct s_instr *instr){}
guint clb_leq(struct s_instr *instr){}
guint clb_geq(struct s_instr *instr){}
guint clb_add(struct s_instr *instr){}
guint clb_sub(struct s_instr *instr){}
guint clb_mul(struct s_instr *instr){}
guint clb_div(struct s_instr *instr){}
guint clb_uneg(struct s_instr *instr){}
