/*      Copyright (C) 2002 Guillaume Bour
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *  
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *  
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 *      MA 02111-1307, USA.
 */
  
  %{
    #include <stdio.h>
    #include <gtk/gtk.h>
    #include "turtle.h"
    #include "middle-code.h"
    #include "op.h"

    guint linecnt;
    gchar *input_str;
    guint curpos;

    extern struct s_turtle my_turtle;

    typedef int(*lexlang_t)(void);
    lexlang_t lexlang[2] = {
      frlex,
      enlex
    };

  %}

%union {
  guint intval;
  gdouble floatval;
  gboolean boolval;
  gchar *strval;

  GSList *list;
  struct s_mc *mc;
}

%token            TOK_BEGPROC      TOK_ENDPROC
%token <strval>   TOK_PROCNAME     TOK_VARIABLE
%token            TOK_MOVEFORWARD  TOK_MOVEBACK 
%token            TOK_TURNRIGHT    TOK_TURNLEFT
%token            TOK_SETPOS       TOK_SETHEADING
%token            TOK_CLEARSCREEN
%token            TOK_HIDETURTLE   TOK_SHOWTURTLE
%token            TOK_COLOR
%token            TOK_NOTRACE      TOK_TRACE
%token            TOK_WRITE        TOK_STOP
%token            TOK_RETURN
%token            TOK_AFFECT       TOK_LOCALAFFECT 
%token            TOK_DANCE        TOK_SLEEP

%token            TOK_REPEAT       TOK_IF
%token            TOK_WHILE

%token <intval>   TOK_INT
%token <floatval> TOK_FLOAT
%token <boolval>  TOK_BOOL
%token <strval>   TOK_STRING
%token            TOK_LOOPCNT      TOK_HAZARD
%token            TOK_GETHEADING

%type  <list>     arguments
%type  <mc>       arith_expr       bool_expr
%type  <mc>       expression

%token            TOK_LEQ          TOK_GEQ


%left '+' '-'
%left '*' '/'
%left uneg
%left '(' ')'
%%

input:         procedures statements
;

procedures:
             | procedure procedures
;

procedure:     TOK_BEGPROC TOK_PROCNAME 
                 { mc_beginning_procedure($2); }
               parameters statements TOK_ENDPROC
                 { mc_ending_procedure(); }
;

parameters:
             | TOK_VARIABLE
                 { mc_add_proc_parameter($1); }
               parameters
;

statements:
             | statement statements
;

statement:     proc_call
             | repeat_expr
             | if_expr
             | while_expr
	     
             | TOK_MOVEFORWARD arith_expr
                 { mc_add_unary_op(FALSE, OP_MOVEFORWARD, $2); }
             | TOK_MOVEBACK    arith_expr
                 { mc_add_unary_op(FALSE, OP_MOVEBACK, $2); }
             | TOK_TURNRIGHT   arith_expr
                 { mc_add_unary_op(FALSE, OP_TURNRIGHT, $2); }
             | TOK_TURNLEFT    arith_expr
                 { mc_add_unary_op(FALSE, OP_TURNLEFT, $2); }
             | TOK_SETPOS      arith_expr arith_expr
                 { mc_add_binary_op(FALSE, OP_SETPOS, $2, $3); }
             | TOK_SETHEADING  arith_expr
                 { mc_add_unary_op(FALSE, OP_SETHEADING, $2); }
             | TOK_CLEARSCREEN
                 { mc_add_noarg_op(FALSE, OP_CLEARSCREEN); }
             | TOK_HIDETURTLE
                 { mc_add_noarg_op(FALSE, OP_HIDETURTLE); }
             | TOK_SHOWTURTLE
                 { mc_add_noarg_op(FALSE, OP_SHOWTURTLE); }
             //vrifier que l'interval est OK (0  15) =>  l'exec
             | TOK_COLOR arith_expr
                 { mc_add_unary_op(FALSE, OP_SETPREDEFCOLOR, $2); }
             //vrifier que l'interval est OK (0  255) =>  l'exec
             | TOK_COLOR '[' arith_expr arith_expr arith_expr ']'
                 { 
		   GSList *colors = NULL;
		   struct s_mc *mccolors;

		   colors = g_slist_append(colors, $3);
		   colors = g_slist_append(colors, $4);
		   colors = g_slist_append(colors, $5);

		   mccolors = mc_alloc(MC_LIST, colors);
		   mc_add_unary_op(FALSE, OP_SETUSERDEFCOLOR, mccolors);
		 }
             | TOK_NOTRACE
                 { mc_add_noarg_op(FALSE, OP_NOTRACE); }
             | TOK_TRACE
                 { mc_add_noarg_op(FALSE, OP_TRACE); }
             | TOK_WRITE       expression
                 { mc_add_unary_op(FALSE, OP_WRITE, $2); }
             | TOK_STOP
                 { mc_add_noarg_op(FALSE, OP_STOP); }
             | TOK_RETURN      expression
                 { mc_add_unary_op(FALSE, OP_RETURN, $2); }
             | TOK_AFFECT      TOK_STRING expression
                 { 
		   struct s_mc *var = mc_add_literal(T_STRING, $2);
		   mc_add_binary_op(FALSE, OP_AFFECT, var, $3); 
		 }
             | TOK_LOCALAFFECT TOK_STRING expression
                 { 
		   struct s_mc *var = mc_add_literal(T_STRING, $2);
		   mc_add_binary_op(FALSE, OP_LOCAFFECT, var, $3); 
		 }

             | TOK_DANCE
                 { mc_add_noarg_op(FALSE, OP_DANCE); }
             | TOK_SLEEP
                 { mc_add_noarg_op(FALSE, OP_SLEEP); }
;

proc_call:     TOK_PROCNAME 
                 { 
		   /* we get the pointer onto the procedure (by its name) */
		   $<mc>$ = mc_alloc(MC_REF, mc_find_procedure($1));
		 }
               arguments
                 { 
		   struct s_mc *plist;

		   plist = $3 == NULL?NULL:mc_alloc(MC_REF, (gpointer) $3); 
		   //~~~
		   mc_add_binary_op(FALSE, OP_PROCCALL, $<mc>2, plist); 
		 }
;

arguments:       { $$ = NULL; }
             | expression arguments
                 { $$ = g_slist_prepend($2, $1); }
;

repeat_expr:   TOK_REPEAT 
                 { $<mc>$ = mc_add_noarg_op(TRUE, OP_NOOP); }
               arith_expr 
                 { 
		   /* warning: the condition is the third argument !! */
		   $<mc>$ = mc_add_binary_op(TRUE, OP_REPEAT, NULL, $3); 
		 } 
              '[' statements ']'
                 {
		   struct s_mc *jmp, *noop;

		   jmp = mc_add_unary_op(TRUE, OP_JUMP, NULL);
		   mc_update_branch((struct s_instr *)jmp->arg,
				    (struct s_instr *)($<mc>2)->arg);

		   noop = mc_add_noarg_op(TRUE, OP_NOOP);
		   mc_update_branch((struct s_instr *)($<mc>4)->arg, 
				    (struct s_instr *)noop->arg);

		   mc_free($<mc>2);
		   mc_free($<mc>4);
		   mc_free(jmp);
		   mc_free(noop);
		 }
;

if_expr:       TOK_IF bool_expr '['  statements ']' else_expr
;

else_expr:   
             | '[' statements ']'
;

while_expr:    TOK_WHILE bool_expr '[' statements ']'
;

expression:    TOK_STRING
                 { $$ = mc_add_literal(T_STRING, $1); }
             | bool_expr
             | arith_expr
;

bool_expr:     TOK_BOOL
                 { $$ = mc_add_literal(T_BOOL, $1); }

             | bool_expr '&' bool_expr
             | bool_expr '|' bool_expr

             | arith_expr '=' arith_expr
             | arith_expr '<' arith_expr
             | arith_expr '>' arith_expr
             | arith_expr TOK_LEQ arith_expr
             | arith_expr TOK_GEQ arith_expr
;

arith_expr:    TOK_INT
                 { $$ = mc_add_literal(T_INT, $1); }
             | TOK_FLOAT
                 { $$ = mc_add_literal(T_FLOAT, $1); }
             | TOK_VARIABLE
                 { $$ = mc_add_variable($1); }
             | TOK_LOOPCNT
                 { 
		   /* we must get the reference to the loop counter !!! */
		   $$ = mc_add_noarg_op(TRUE, OP_LOOPCNT); 
		 }
             | TOK_HAZARD     arith_expr
                 { $$ = mc_add_unary_op(TRUE, OP_HAZARD, $2); }
             | TOK_GETHEADING
                 { $$ = mc_add_noarg_op(TRUE, OP_GETHEADING); }

             | arith_expr '+' arith_expr
             | arith_expr '-' arith_expr
             | arith_expr '*' arith_expr
             | arith_expr '/' arith_expr
             | '-' arith_expr %prec uneg
                 { $$ = NULL; }
             | '(' arith_expr ')'
                 { $$ = NULL; }
;             
%%

int  launch_parser(gchar *program)
{
  input_str = program;
  curpos = 0;
  linecnt = 0;

  yyparse();
  
  return(0);
}

void yyerror(char *error)
{  
  printf("%d: invalid logo code\n", linecnt); 
}

/* here we switch between differents languages-parser */
int yylex()
{
  return((*lexlang[my_turtle.language])());
}
