/**
 *  @file    proc.c      
 *  @author  Guillaume Bour. 2002
 
 *  @version 0.1
 *    @date  08/02/2002

 *  procedures management
 */
/*      Copyright (C) 2002 Guillaume Bour
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *  
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *  
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 *      MA 02111-1307, USA.
 */
#include <stdio.h>
#include <gtk/gtk.h>
#include "turtle.h"
#include "proc.h"
#include "instr.h"
#include "variable.h"
#include "debug.h"

extern struct s_turtle my_turtle;

/**
   Procedure allocation.<br>

   <b>visibility :: <i>public</i></b>

   @remarks:
            1. the instructions, parameters & variables lists are set to NULL
	    2. the return value is set to NULL

   @param   name                        the procedure name

   @return  the allocated procedure
 */
struct s_proc *proc_alloc(gchar *name)
{
  struct s_proc *myvar = g_new(struct s_proc, 1);

  myvar->name = g_strdup(name);
  myvar->instructions = NULL;
  myvar->parameters = NULL;
  myvar->variables = NULL;
  myvar->retval = NULL;
  
  return(myvar);
}


/**
   Procedure freeing.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. the second argument is not used (just for GTK compatibility)
	    2. we firstly free the insternal lists 
               (instructions, parameters & variables)

   @param   elt                         the procedure to free
   @param   nil                         not used
   @return  <none>
 */
void proc_free(gpointer elt, gpointer nil)
{
  register struct s_proc *proc =(struct s_proc *)elt;

  g_slist_foreach(proc->instructions, instr_free, NULL);
  g_slist_free(proc->instructions);

  g_slist_foreach(proc->parameters, var_free, NULL);
  g_slist_free(proc->parameters);

  g_slist_foreach(proc->variables, var_free, NULL);
  g_slist_free(proc->variables);

  g_free(proc->name);
  g_free(proc->retval);
  g_free(proc);
}


/**
   Procedure comparison.<br>

   <b>visibility :: <i>public</i></b>

   @param   myproc                      the procedure
   @param   name                        the searched procedure name

   @return  0 is the procedure <i>myproc</i> is named <i>name</i>,
            not 0 else.
 */
gint proc_comparison(gconstpointer myproc, gconstpointer name)
{
  return(strcmp(((struct s_proc *)myproc)->name, (gchar *)name));
}

#ifdef DEBUG1
void proc_debug(gpointer elt, gpointer dec)
{
  gint pos;
  gchar txt[100];
  register struct s_proc *proc =(struct s_proc *)elt;

  debug_append2("==> Procedure(*%d: %s)\n  :parameters\n", proc, proc->name);
  g_slist_foreach(proc->parameters, var_debug, NULL);

  debug_append("\n  :variables\n");
  g_slist_foreach(proc->variables, var_debug, NULL);

  debug_append("\n  :instructions\n");
  g_slist_foreach(proc->instructions, instr_debug, NULL);

  debug_append("\n\n");
}
#endif
