/**
 *  @file    variable.c
 *  @author  Guillaume Bour. 2002
 
 *  @version 0.1
 *    @date  05/02/2002

 *  variables management
 */
/*      Copyright (C) 2002 Guillaume Bour
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *  
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *  
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 *      MA 02111-1307, USA.
 */
#include <stdio.h>
#include <gtk/gtk.h>
#include "variable.h"

#ifdef DEBUG1
  #include "turtle.h"
  extern struct s_turtle my_turtle;
#endif

/**
   Variable allocation.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. the value is set to <null>

   @param   name                        the variable name
   @return  the allocated variable
 */
struct s_variable *var_alloc(gchar *name, type_t type)
{
  struct s_variable *myvar = g_new(struct s_variable, 1);

  myvar->name = g_strdup(name);
  myvar->type = type;
  myvar->value = NULL;
  
  return(myvar);
}


/**
   Variable freeing.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. the second argument is not used (just for GTK compatibility)

   @param   elt                         the variable to free
   @param   nil                         not used
   @return  <none>
 */
void var_free(gpointer elt, gpointer nil)
{
  register struct s_variable *var =(struct s_variable *)elt;

  g_free(var->name);
  /* if NULL, do nothing */
  g_free(var->value);
  g_free(var);
}



/**
   Variable comparison.<br>

   <b>visibility :: <i>public</i></b>

   @param   myvar                       the variable
   @param   name                        the searched variable name

   @return  0 is the variable <i>myvar</i> is named <i>name</i>,
            not 0 else.
 */
gint var_comparison(gconstpointer myvar, gconstpointer name)
{
  return(strcmp(((struct s_variable *)myvar)->name, 
		(gchar *)name));
}


#ifdef DEBUG1
void var_debug(gpointer elt, gpointer dec)
{
  gint pos;
  register struct s_variable *var =(struct s_variable *)elt;

  pos = -1;
  gtk_editable_insert_text(GTK_EDITABLE(my_turtle.debug_txt),"    (", 5, &pos);
  pos = -1;
  gtk_editable_insert_text(GTK_EDITABLE(my_turtle.debug_txt), 
			   var->name, strlen(var->name), &pos);
  pos = -1;
  gtk_editable_insert_text(GTK_EDITABLE(my_turtle.debug_txt), 
			   ")\n", 2, &pos);
}
#endif
